# automatically generated by the FlatBuffers compiler, do not modify

# namespace: flatbuf

# A concrete bound, which can be an expression or unbounded
class ConcreteBoundImpl(object):
    NONE = 0
    Expression = 1
    Unbounded = 2

