# automatically generated by the FlatBuffers compiler, do not modify

# namespace: flatbuf

class JoinKind(object):
    Anti = 0
    Cross = 1
    FullOuter = 2
    Inner = 3
    LeftOuter = 4
    LeftSemi = 5
    RightOuter = 6

