# automatically generated by the FlatBuffers compiler, do not modify

# namespace: flatbuf

class LiteralImpl(object):
    NONE = 0
    BooleanLiteral = 1
    Int8Literal = 2
    Int16Literal = 3
    Int32Literal = 4
    Int64Literal = 5
    UInt8Literal = 6
    UInt16Literal = 7
    UInt32Literal = 8
    UInt64Literal = 9
    DateLiteral = 10
    TimeLiteral = 11
    TimestampLiteral = 12
    IntervalLiteral = 13
    DurationLiteral = 14
    DecimalLiteral = 15
    Float16Literal = 16
    Float32Literal = 17
    Float64Literal = 18
    ListLiteral = 19
    StructLiteral = 20
    MapLiteral = 21
    StringLiteral = 22
    BinaryLiteral = 23
    FixedSizeBinaryLiteral = 24

