# automatically generated by the FlatBuffers compiler, do not modify

# namespace: flatbuf

class SparseTensorIndex(object):
    NONE = 0
    SparseTensorIndexCOO = 1
    SparseMatrixIndexCSX = 2
    SparseTensorIndexCSF = 3

