# automatically generated by the FlatBuffers compiler, do not modify

# namespace: tabular

import flatbuffers
from flatbuffers.compat import import_numpy
np = import_numpy()

class CreateViewRequest(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAs(cls, buf, offset=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = CreateViewRequest()
        x.Init(buf, n + offset)
        return x

    @classmethod
    def GetRootAsCreateViewRequest(cls, buf, offset=0):
        """This method is deprecated. Please switch to GetRootAs."""
        return cls.GetRootAs(buf, offset)
    # CreateViewRequest
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # CreateViewRequest
    def ViewMetadataArrowBuffer(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            a = self._tab.Vector(o)
            return self._tab.Get(flatbuffers.number_types.Uint8Flags, a + flatbuffers.number_types.UOffsetTFlags.py_type(j * 1))
        return 0

    # CreateViewRequest
    def ViewMetadataArrowBufferAsNumpy(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            return self._tab.GetVectorAsNumpy(flatbuffers.number_types.Uint8Flags, o)
        return 0

    # CreateViewRequest
    def ViewMetadataArrowBufferLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # CreateViewRequest
    def ViewMetadataArrowBufferIsNone(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        return o == 0

    # CreateViewRequest
    def ViewDataArrowSchema(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            a = self._tab.Vector(o)
            return self._tab.Get(flatbuffers.number_types.Uint8Flags, a + flatbuffers.number_types.UOffsetTFlags.py_type(j * 1))
        return 0

    # CreateViewRequest
    def ViewDataArrowSchemaAsNumpy(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            return self._tab.GetVectorAsNumpy(flatbuffers.number_types.Uint8Flags, o)
        return 0

    # CreateViewRequest
    def ViewDataArrowSchemaLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # CreateViewRequest
    def ViewDataArrowSchemaIsNone(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        return o == 0

def Start(builder): builder.StartObject(2)
def CreateViewRequestStart(builder):
    """This method is deprecated. Please switch to Start."""
    return Start(builder)
def AddViewMetadataArrowBuffer(builder, viewMetadataArrowBuffer): builder.PrependUOffsetTRelativeSlot(0, flatbuffers.number_types.UOffsetTFlags.py_type(viewMetadataArrowBuffer), 0)
def CreateViewRequestAddViewMetadataArrowBuffer(builder, viewMetadataArrowBuffer):
    """This method is deprecated. Please switch to AddViewMetadataArrowBuffer."""
    return AddViewMetadataArrowBuffer(builder, viewMetadataArrowBuffer)
def StartViewMetadataArrowBufferVector(builder, numElems): return builder.StartVector(1, numElems, 1)
def CreateViewRequestStartViewMetadataArrowBufferVector(builder, numElems):
    """This method is deprecated. Please switch to Start."""
    return StartViewMetadataArrowBufferVector(builder, numElems)
def AddViewDataArrowSchema(builder, viewDataArrowSchema): builder.PrependUOffsetTRelativeSlot(1, flatbuffers.number_types.UOffsetTFlags.py_type(viewDataArrowSchema), 0)
def CreateViewRequestAddViewDataArrowSchema(builder, viewDataArrowSchema):
    """This method is deprecated. Please switch to AddViewDataArrowSchema."""
    return AddViewDataArrowSchema(builder, viewDataArrowSchema)
def StartViewDataArrowSchemaVector(builder, numElems): return builder.StartVector(1, numElems, 1)
def CreateViewRequestStartViewDataArrowSchemaVector(builder, numElems):
    """This method is deprecated. Please switch to Start."""
    return StartViewDataArrowSchemaVector(builder, numElems)
def End(builder): return builder.EndObject()
def CreateViewRequestEnd(builder):
    """This method is deprecated. Please switch to End."""
    return End(builder)