# automatically generated by the FlatBuffers compiler, do not modify

# namespace: flatbuf

class IntervalLiteralImpl(object):
    NONE = 0
    IntervalLiteralMonths = 1
    IntervalLiteralDaysMilliseconds = 2

